<?php

namespace Drupal\mapbox_field\Plugin\Field\FieldFormatter;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mapbox\Mapbox;

/**
 * Plugin implementation of the 'mapbox_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "mapbox_formatter",
 *   label = @Translation("Mapbox"),
 *   field_types = {
 *     "mapbox_field"
 *   }
 * )
 */
class MapboxFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\mapbox\Mapbox definition.
   *
   * @var \Drupal\mapbox\Mapbox
   */
  protected $mapbox;

  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, Mapbox $mapbox) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->mapbox = $mapbox;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('mapbox')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      // Implement settings form.
    ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // Implement settings summary.

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $accessToken  = $this->mapbox->accessToken();
    $style        = $this->mapbox->getStyle();

    $elements = [];

    foreach ($items as $delta => $item) {
      $instance_num   = 0;
      $instance_num   = $delta + 1;
      $instance_delta = 'mapbox-'.$items[$delta]->getEntity()->id().$delta.$instance_num;
      $instance_marker = 'mapbox-marker-'.$items[$delta]->getEntity()->id().$delta.$instance_num;

      $mapbox['accessToken'] = $accessToken;
      $mapbox['style']       = $style;
      $mapbox['options'] = [
        'finalRender' => TRUE,
        'maxPitch'    => (int) $items[$delta]->maxPitch,
        'keyboard'    => (int) $items[$delta]->keyboard,
        'minZoom'     => (int) $items[$delta]->minZoom,
        'maxZoom'     => (int) $items[$delta]->maxZoom,
        'threeD'      => (int) $items[$delta]->threeDBuildings,
        'marked'      => (int) $items[$delta]->marked,
        'pitch'       => (int) $items[$delta]->pitch,
        'zoom'        => (int) $items[$delta]->zoom,
      ];
      $mapbox['position'] = [
        'marker'          => $items[$delta]->marker ? $items[$delta]->marker : '/'.drupal_get_path('module', 'mapbox_field').'/marker.png',
        'markerText'      => $items[$delta]->markerText ?? $items[$delta]->markerText,
        'lat'             => (float) $items[$delta]->lat,
        'lng'             => (float) $items[$delta]->lng,
        'instance_delta'  => $instance_delta,
        'instance_marker' => $instance_marker
      ];

      $element['#theme']  = 'mapbox_field';
      $element['#mapbox'] = $mapbox['position'];
      $element['#attached']['library'][] = 'mapbox/renderer';
      $element['#attached']['library'][] = 'mapbox/apis';
      $element['#attached']['drupalSettings']['mapbox_renderer'][] = $mapbox;

      $elements[$delta] = $element;
    }

    return $elements;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    // The text value has no text format assigned to it, so the user input
    // should equal the output, including newlines.
    return nl2br(Html::escape($item->value));
  }

}
