<?php

namespace Drupal\mapbox_field\Plugin\Field\FieldFormatter;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mapbox\Mapbox;

/**
 * Plugin implementation of the 'mapbox_single' formatter.
 *
 * @FieldFormatter(
 *   id = "mapbox_single",
 *   label = @Translation("Mapbox - All in one"),
 *   field_types = {
 *     "mapbox_field"
 *   }
 * )
 */
class MapboxSingle extends FormatterBase {

  /**
   * Drupal\mapbox\Mapbox definition.
   *
   * @var \Drupal\mapbox\Mapbox
   */
  protected $mapbox;

  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, Mapbox $mapbox) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->mapbox = $mapbox;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('mapbox')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'threeDBuildings' => 0,
      'navigation'      => 0,
      'keyboard'        => 0,
      'minZoom'         => 3,
      'maxZoom'         => 15,
      'maxPitch'        => 65,
      'pitch'           => 45,
      'zoom'            => 9
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['threeDBuildings'] = [
      '#title' => $this->t('3D Buildings'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('threeDBuildings'),
    ];
    $form['keyboard'] = [
      '#title' => $this->t('Keyboard Controls'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('keyboard'),
    ];
    $form['navigation'] = [
      '#title' => $this->t('Navigation Controls'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('navigation'),
    ];
    $form['minZoom'] = [
      '#title' => $this->t('Min Zoom'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('minZoom'),
      '#attributes' => [
        'min' => 3,
        'max' => 18
      ]
    ];
    $form['maxZoom'] = [
      '#title' => $this->t('Max Zoom'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('maxZoom'),
      '#attributes' => [
        'min' => 5,
        'max' => 18
      ]
    ];
    $form['maxPitch'] = [
      '#title' => $this->t('Max Pitch'),
      '#type' => 'select',
      '#options' => [30 => 30, 35 => 35, 40 => 40, 45 => 45, 50 => 50, 55 => 55, 60 => 60, 65 => 65],
      '#default_value' => $this->getSetting('maxPitch'),
    ];
    $form['pitch'] = [
      '#title' => $this->t('Default Pitch'),
      '#type' => 'select',
      '#options' => [30 => 30, 35 => 35, 40 => 40, 45 => 45, 50 => 50, 55 => 55, 60 => 60, 65 => 65],
      '#default_value' => $this->getSetting('pitch'),
    ];
    $form['zoom'] = [
      '#title' => $this->t('Default Zoom'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('zoom'),
      '#attributes' => [
        'min' => 3,
        'max' => 18
      ]
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Shows markers from multi-valued field in one map.');

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $mapbox['mapboxInstance'] = 'mapbox-map-'.rand(1000000, 999999);
    $mapbox['accessToken']    = $this->mapbox->accessToken();
    $mapbox['style']          = $this->mapbox->getStyle();
    $mapbox['options']  = [
      'threeDBuildings' => (int) $this->getSetting('threeDBuildings'),
      'navigation'      => (int) $this->getSetting('navigation'),
      'maxPitch'        => (int) $this->getSetting('maxPitch'),
      'keyboard'        => (int) $this->getSetting('keyboard'),
      'minZoom'         => (int) $this->getSetting('minZoom'),
      'maxZoom'         => (int) $this->getSetting('maxZoom'),
      'pitch'           => (int) $this->getSetting('pitch'),
      'zoom'            => (int) $this->getSetting('zoom'),
    ];

    foreach ($items as $delta => $item) {
      $instance_num = $delta + 1;
      $positions[]  = [
        'lat'            => (float) $items[$delta]->lat,
        'lng'            => (float) $items[$delta]->lng,
        'marker'         => $items[$delta]->marker ? $items[$delta]->marker : '/'.drupal_get_path('module', 'mapbox').'/marker.png',
        'markerText'     => $items[$delta]->markerText ?? $items[$delta]->markerText,
        'markerInstance' => $items[$delta]->getEntity()->id().$delta.$instance_num
      ];
    }

    $mapbox['positions'] = $positions;
    $mapbox['center']    = $positions[0];

    $build = [
      '#theme'          => 'mapbox_field_singlemap',
      '#positions'      => $mapbox['positions'],
      '#mapboxInstance' => $mapbox['mapboxInstance']
    ];
    $build['#attached']['library'][] = 'mapbox/singlemap';
    $build['#attached']['library'][] = 'mapbox/apis';
    $build['#attached']['drupalSettings']['mapbox_singlemap'] = $mapbox;

    $elements[] = $build;
    return $elements;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    // The text value has no text format assigned to it, so the user input
    // should equal the output, including newlines.
    return nl2br(Html::escape($item->value));
  }

}
