<?php

namespace Drupal\mapbox_field\Plugin\Field\FieldWidget;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mapbox\Mapbox;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Plugin implementation of the 'mapbox_widget' widget.
 *
 * @FieldWidget(
 *   id = "mapbox_widget",
 *   module = "mapbox_field",
 *   label = @Translation("Mapbox Widget"),
 *   field_types = {
 *     "mapbox_field"
 *   }
 * )
 */
class MapboxWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\mapbox\Mapbox definition.
   *
   * @var \Drupal\mapbox\Mapbox
   */
  protected $mapbox;
  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, Mapbox $mapbox) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->mapbox = $mapbox;
  }
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      [],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('mapbox')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $accessToken = $this->mapbox->accessToken();
    $style = $this->mapbox->getStyle();
    $instance_num = $delta + 1;
    $instance_delta = 'mapbox-'.$items[$delta]->getEntity()->id().$delta.$instance_num;
    $instance_marker = 'mapbox-marker-'.$items[$delta]->getEntity()->id().$delta.$instance_num;

    $mapbox['accessToken'] = $accessToken;
    $mapbox['style']       = $style;
    $mapbox['options'] = [
      'finalRender' => TRUE,
      'maxPitch'    => (int) $items[$delta]->maxPitch,
      'keyboard'    => (int) $items[$delta]->keyboard,
      'minZoom'     => (int) $items[$delta]->minZoom,
      'maxZoom'     => (int) $items[$delta]->maxZoom,
      'threeD'      => (int) $items[$delta]->threeDBuildings,
      'marked'      => (int) $items[$delta]->marked,
      'pitch'       => (int) $items[$delta]->pitch,
      'zoom'        => (int) $items[$delta]->zoom,
    ];
    $mapbox['position'] = [
      'marker'          => $items[$delta]->marker ? $items[$delta]->marker : '/'.drupal_get_path('module', 'mapbox_field').'/marker.png',
      'markerText'      => $items[$delta]->markerText ?? $items[$delta]->markerText,
      'lat'             => (float) $items[$delta]->lat,
      'lng'             => (float) $items[$delta]->lng,
      'instance_delta'  => $instance_delta,
      'instance_marker' => $instance_marker
    ];

    $element += [
      '#type' => 'fieldset',
      '#title' => $this->t('Map'),
    ];

    $element['#attached']['library'][] = 'mapbox/widget';
    $element['#attached']['library'][] = 'mapbox/apis';
    $element['#attached']['drupalSettings']['mapbox_renderer'][$instance_delta] = $mapbox;

    if (!$accessToken)
      $noTokenMessage = $this->t('Missing Mapboxs\' access token. Please provide the access token in <a href="@url" target="_blank">settings</a>.',[
        '@url' => Url::fromRoute('mapbox_field.config_form', [])->toString()
      ]);

    $element['lat'] = [
      '#title' => $this->t('Latitude'),
      '#type' => 'textfield',
      '#size' => 18,
      '#default_value' => isset($items[$delta]->lat) ? $items[$delta]->lat : NULL,
      '#attributes' => [
        'data-lat-delta' => $instance_delta,
      ],
      '#prefix' => '<div class="column">'
    ];
    $element['lng'] = [
      '#title' => $this->t('Longitude'),
      '#type' => 'textfield',
      '#size' => 18,
      '#default_value' => isset($items[$delta]->lng) ? $items[$delta]->lng : NULL,
      '#attributes' => [
        'data-lng-delta' => $instance_delta,
      ],
    ];
    $element['markerText'] = [
      '#title' => $this->t('Marker Text'),
      '#type' => 'textfield',
      '#size' => 18,
      '#default_value' => isset($items[$delta]->markerText) ? $items[$delta]->markerText : NULL,
      '#attributes' => [
        'data-marker-delta' => $instance_delta,
      ],
    ];
    $element['marker'] = [
      '#title' => $this->t('Marker'),
      '#type' => 'textfield',
      '#description' => $this->t('Marker needs to be uploaded on your site first. Give a relative path like /sites/default/files/marker.png'),
      '#size' => 18,
      '#default_value' => isset($items[$delta]->marker) ? $items[$delta]->marker : NULL,
      '#attributes' => [
        'data-marker-delta' => $instance_delta,
      ],
    ];
    $element['threeDBuildings'] = [
      '#title' => $this->t('3D Buildings'),
      '#type' => 'checkbox',
      '#size' => 18,
      '#default_value' => isset($items[$delta]->threeDBuildings) ? $items[$delta]->threeDBuildings : NULL,
      '#attributes' => [
        'data-3d-delta' => $instance_delta,
      ],
    ];
    $element['keyboard'] = [
      '#title' => $this->t('Keyboard Controls'),
      '#type' => 'checkbox',
      '#size' => 18,
      '#default_value' => isset($items[$delta]->keyboard) ? $items[$delta]->keyboard : NULL,
      '#attributes' => [
        'data-keyboard-delta' => $instance_delta,
      ],
    ];
    $element['minZoom'] = [
      '#title' => $this->t('Min Zoom'),
      '#type' => 'select',
      '#options' => [3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10, 11 => 11, 12 => 12, 13 => 13, 14 => 14, 15 => 15, 16 => 16, 17 => 17, 18 => 18],
      '#default_value' => isset($items[$delta]->minZoom) ? $items[$delta]->minZoom : 3,
      '#attributes' => [
        'data-minZoom-delta' => $instance_delta,
      ],
      '#prefix' => '<div class="inner-wrapper"><div class="column">'
    ];
    $element['maxZoom'] = [
      '#title' => $this->t('Max Zoom'),
      '#type' => 'select',
      '#options' => [5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10, 11 => 11, 12 => 12, 13 => 13, 14 => 14, 15 => 15, 16 => 16, 17 => 17, 18 => 18, 19 => 19, 20 => 20],
      '#default_value' => isset($items[$delta]->maxZoom) ? $items[$delta]->maxZoom : 18,
      '#attributes' => [
        'data-maxZoom-delta' => $instance_delta,
      ],
      '#suffix' => '</div>'
    ];
    $element['maxPitch'] = [
      '#title' => $this->t('Max Pitch'),
      '#type' => 'select',
      '#options' => [30 => 30, 35 => 35, 40 => 40, 45 => 45, 50 => 50, 55 => 55, 60 => 60, 65 => 65],
      '#default_value' => isset($items[$delta]->maxPitch) ? $items[$delta]->maxPitch : 65,
      '#attributes' => [
        'data-maxPitch-delta' => $instance_delta,
      ],
      '#prefix' => '<div class="column">'
    ];
    $element['pitch'] = [
      '#title' => $this->t('Pitch'),
      '#type' => 'select',
      '#options' => [30 => 30, 35 => 35, 40 => 40, 45 => 45, 50 => 50, 55 => 55, 60 => 60, 65 => 65],
      '#default_value' => isset($items[$delta]->pitch) ? $items[$delta]->pitch : 45,
      '#attributes' => [
        'data-pitch-delta' => $instance_delta,
      ],
      '#suffix' => '</div></div>'
    ];
    $element['zoom'] = [
      '#type' => 'hidden',
      '#default_value' => isset($items[$delta]->zoom) ? $items[$delta]->zoom : 9,
      '#attributes' => [
        'data-zoom-delta' => $instance_delta,
      ],
    ];
    $element['marked'] = [
      '#type' => 'hidden',
      '#default_value' => isset($items[$delta]->marked) ? $items[$delta]->marked : 0,
      '#attributes' => [
        'data-marked-delta' => $instance_delta,
      ],
      '#suffix' => '</div>'
    ];
    $element['mapbox_map'] = [
      '#type' => 'item',
      '#title' => $this->t('Mark Location'),
      '#markup' => '<div class="mapbox-map" id="'.$instance_delta.'">'.$noTokenMessage.'<div class="mapbox-marker" id="'.$instance_marker.'"></div></div>',
      '#prefix' => '<div class="column">',
    ];
    $element['remove_btn_'.$instance_num] = [
      '#type' => 'button',
      '#value' => $this->t('Remove'),
      '#attributes' => [
        'data-remove-delta' => $instance_delta,
        'class' => ['mapbox-remover']
      ],
      '#suffix' => '</div>'
    ];

    return $element;
  }

}
